@@ -134,11 +134,11 @@ sub execute_stacks {
         $path = $sample->{'path'} . $sample->{'file'} . "." . $sample->{'suffix'};
         
         if ($sample->{'type'} eq "sample") {
-            $cmd = $exe_path . "ustacks -t $sample->{'fmt'} -f $path -o $out_path -i $sample_id -r $minc "  . join(" ", @_ustacks) . " 2>&1";
+            $cmd = $exe_path . "ustacks -t $sample->{'fmt'} -f $path -o $out_path -i $sample_id $minc "  . join(" ", @_ustacks) . " 2>&1";
         } elsif ($sample->{'type'} eq "parent") {
-            $cmd = $exe_path . "ustacks -t $sample->{'fmt'} -f $path -o $out_path -i $sample_id -r $minc "  . join(" ", @_ustacks) . " 2>&1";
+            $cmd = $exe_path . "ustacks -t $sample->{'fmt'} -f $path -o $out_path -i $sample_id $minc "  . join(" ", @_ustacks) . " 2>&1";
         } elsif ($sample->{'type'} eq "progeny") {
-            $cmd = $exe_path . "ustacks -t $sample->{'fmt'} -f $path -o $out_path -i $sample_id -r $minrc " . join(" ", @_ustacks) . " 2>&1";
+            $cmd = $exe_path . "ustacks -t $sample->{'fmt'} -f $path -o $out_path -i $sample_id $minrc " . join(" ", @_ustacks) . " 2>&1";
         }
         print STDERR  "  $cmd\n";
         print $log_fh "$cmd\n";
@@ -891,6 +891,11 @@ sub parse_command_line {
 	usage();
     }
 
+    if (scalar(@parents) == 0 && scalar(@samples) == 0 && length($popmap_path) > 0 && length($sample_path) == 0) {
+	print STDERR "If you are using a population map to specify samples, you must specify the path to the directory containing the samples (--samples).\n";
+	usage();
+    }
+
     if (length($sample_path) > 0) {
         $sample_path .= "/" if (substr($sample_path, -1) ne "/");
     }
