@@ -148,8 +148,8 @@ sub execute_stacks {
             #
             # Pull the depth of coverage from pstacks.
             #
-            my @lines   = grep(/Mean coverage depth is/, @results);
-            my ($depth) = ($lines[0] =~ /^  Mean coverage depth is (\d+\.?\d*); Std Dev: .+; Max: .+$/);
+            my @lines   = grep(/mean coverage/, @results);
+            my ($depth) = ($lines[-1] =~ /^Kept \d+ loci; mean coverage is ([\d.]+) \(stdev: [\d.]+, max: [\d.]+\).$/);
             push(@depths_of_cov, [$sample->{'file'}, $depth]);
         }
         write_results(\@results, $log_fh);
@@ -850,6 +850,11 @@ sub parse_command_line {
 	usage();
     }
 
+    if (scalar(@parents) == 0 && scalar(@samples) == 0 && length($popmap_path) > 0 && length($sample_path) == 0) {
+	print STDERR "If you are using a population map to specify samples, you must specify the path to the directory containing the samples (--samples).\n";
+	usage();
+    }
+
     if (length($sample_path) > 0) {
         $sample_path .= "/" if (substr($sample_path, -1) ne "/");
     }
