@@ -59,7 +59,6 @@ public:
     char          *cigar;         // CIGAR string describing how the datum aligns to the catalog locus.
     double         lnl;           // Log likelihood of this locus.
     vector<char *> obshap;        // Observed Haplotypes
-    vector<SNP *>  snps;          // All calls for this sample for this locus. size() is [len].
     Datum()  {
         this->id            = -1;
         this->corrected     = false;
@@ -75,8 +74,6 @@ public:
     ~Datum() {
         for (uint i = 0; i < this->obshap.size(); i++)
             delete [] this->obshap[i];
-        for (uint i = 0; i < this->snps.size(); i++)
-            delete this->snps[i];
         delete [] this->gtype;
         delete [] this->trans_gtype;
         delete [] this->model;
@@ -165,6 +162,12 @@ PopMap<LocusT>::PopMap(const MetaPopInfo& mpopi, int num_loci)
 
 template<class LocusT>
 PopMap<LocusT>::~PopMap() {
+
+    typename std::map<string, vector<LocusT *> >::iterator it;
+    for (it = this->ordered_loci.begin(); it != this->ordered_loci.end(); it++)
+        it->second.clear();
+    this->ordered_loci.clear();
+    
     for (int i = 0; i < this->num_loci; i++) {
         for (int j = 0; j < metapopinfo.samples().size(); j++)
             delete this->data[i][j];
