@@ -21,6 +21,8 @@
 #ifndef __CATALOG_UTILS_H__
 #define __CATALOG_UTILS_H__
 
+#include <vector>
+#include <string>
 #include <map>
 using std::map;
 #include <set>
@@ -33,6 +35,10 @@ using std::set;
 #include "PopSum.h"
 #include "Vcf.h"
 
+// find_catalogs()
+// Looks for catalog files in the given directory and returns the associated ID(s).
+std::vector<int> find_catalogs(const std::string& dir_path);
+
 int check_whitelist_integrity(map<int, CSLocus *> &, map<int, set<int> > &);
 int reduce_catalog(map<int, CSLocus *> &, set<int> &, set<int> &);
 int reduce_catalog(map<int, CSLocus *> &, map<int, set<int> > &, set<int> &);
