@@ -60,6 +60,7 @@ uint   max_bc_size_2;
 double win_size;
 
 int main (int argc, char* argv[]) {
+    IF_NDEBUG_TRY
 
     parse_command_line(argc, argv);
 
@@ -185,6 +186,7 @@ int main (int argc, char* argv[]) {
          << counters["dis_reads"] << " pairs of reads, " << buf << " clone reads.\n";
 
     return 0;
+    IF_NDEBUG_CATCH_ALL_EXCEPTIONS
 }
 
 int
