@@ -47,6 +47,7 @@ double heterozygote_limit = -3.84;
 double homozygote_limit   =  3.84;
 
 int main (int argc, char* argv[]) {
+    IF_NDEBUG_TRY
 
     parse_command_line(argc, argv);
 
@@ -82,6 +83,7 @@ int main (int argc, char* argv[]) {
     write_sql(merged, unique);
 
     return 0;
+    IF_NDEBUG_CATCH_ALL_EXCEPTIONS
 }
 
 int call_alleles(MergedStack *mtag, vector<DNANSeq *> &reads) {
@@ -199,7 +201,7 @@ int call_consensus(map<int, MergedStack *> &merged, map<int, PStack *> &unique,
             if (invoke_model) {
                 call_alleles(mtag, reads);
 
-                if (model_type == fixed) {
+                if (model_type == ::fixed) {
                     //
                     // Mask nucleotides that are not fixed.
                     //
@@ -499,6 +501,9 @@ int load_radtags(string in_file, HashMap &radtags) {
     while ((c = fh->next_seq()) != NULL) {
         if (i % 10000 == 0) cerr << "Loading aligned sequence " << i << "       \r";
 
+        if (c->aln_type == AlnT::null)
+            continue;
+
         radtags[c->seq].push_back(c);
         i++;
     }
@@ -660,7 +665,7 @@ int parse_command_line(int argc, char* argv[]) {
     if (out_path.at(out_path.length() - 1) != '/')
         out_path += "/";
 
-    if (model_type == fixed && barcode_err_freq == 0) {
+    if (model_type == ::fixed && barcode_err_freq == 0) {
         cerr << "You must specify the barcode error frequency.\n";
         help();
     }
