@@ -72,6 +72,7 @@ map<string, int>           renz_len;
 map<string, map<string, string> > global_dictionary;
 
 int main (int argc, char* argv[]) {
+    IF_NDEBUG_TRY
 
     initialize_renz(renz, renz_cnt, renz_len);
 
@@ -88,10 +89,6 @@ int main (int argc, char* argv[]) {
 
     MetaPopInfo mpopi;
     mpopi.init_directory(in_path);
-    if (mpopi.samples().empty()) {
-        cerr << "Error: Failed to find sample files in directory '" << in_path << "'.\n";
-        return -1;
-    }
 
     if (wl_file.length() > 0) {
         load_marker_list(wl_file, whitelist);
@@ -296,7 +293,9 @@ int main (int argc, char* argv[]) {
     if (out_type == genomic)
         write_genomic(catalog, pmap);
 
+    cerr << "genotypes is done.\n";
     return 0;
+    IF_NDEBUG_CATCH_ALL_EXCEPTIONS
 }
 
 int
@@ -2828,6 +2827,7 @@ int parse_command_line(int argc, char* argv[]) {
             break;
         case 'e':
             enz = optarg;
+            enz.at(0) = tolower(enz.at(0));
             break;
         case 'v':
             version();
@@ -2842,6 +2842,11 @@ int parse_command_line(int argc, char* argv[]) {
         }
     }
 
+    if (optind < argc) {
+        cerr << "Error: Failed to parse command line: '" << argv[optind] << "' is seen as a positional argument. Expected no positional arguments.\n";
+        help();
+    }
+
     if (in_path.length() == 0) {
         cerr << "You must specify a path to the directory containing Stacks output files.\n";
         help();
