@@ -18,8 +18,16 @@
 // along with Stacks.  If not, see <http://www.gnu.org/licenses/>.
 //
 
+#include <ctime>
+#include <fstream>
+#include <sstream>
+#include <iomanip>
+
+#include "constants.h"
 #include "log_utils.h"
 
+using namespace std;
+
 int
 init_log(ofstream &fh, int argc, char **argv)
 {
@@ -44,3 +52,10 @@ init_log(ofstream &fh, int argc, char **argv)
 
     return 0;
 }
+
+std::string as_percentage(double d) {
+    stringstream ss;
+    ss << std::fixed << std::setprecision(1);
+    ss << d*100 << "%";
+    return ss.str();
+}
