@@ -25,9 +25,13 @@
 //
 
 #include <iomanip>
+#include <cstdlib>
 
+#include "constants.h"
 #include "process_radtags.h"
 
+using namespace std;
+
 //
 // Global variables to hold command-line options.
 //
@@ -93,6 +97,7 @@ map<string, int>           renz_cnt;
 map<string, int>           renz_len;
 
 int main (int argc, char* argv[]) {
+    IF_NDEBUG_TRY
 
     initialize_renz(renz, renz_cnt, renz_len);
 
@@ -128,7 +133,7 @@ int main (int argc, char* argv[]) {
             init_adapter_seq(kmer_size, adapter_2, adp_2_len, adp_2_kmers);
         }
 
-        cerr << "    " << distance << " mismatches allowed to adapter sequence.\n";
+        cerr << "    " << ::distance << " mismatches allowed to adapter sequence.\n";
     }
 
     vector<pair<string, string> >        files;
@@ -230,6 +235,7 @@ int main (int argc, char* argv[]) {
     print_results(argc, argv, barcodes, counters, barcode_log);
 
     return 0;
+    IF_NDEBUG_CATCH_ALL_EXCEPTIONS
 }
 
 template <typename fhType>
@@ -576,7 +582,7 @@ process_reads(string prefix,
         }
 
         if (discards && !r->retain)
-            result = out_file_type == FileT::fastq ?
+            result = out_file_type == FileT::fastq || out_file_type == FileT::gzfastq ?
                 write_fastq(discard_fh, s) :
                 write_fasta(discard_fh, s);
 
@@ -687,10 +693,10 @@ process_singlet(Read *href,
         int res = 1;
         if (paired_end == true  && adp_2_len > 0)
             res = filter_adapter_seq(href, adapter_2, adp_2_len, adp_2_kmers,
-                                     kmer_size, distance, len_limit);
+                                     kmer_size, ::distance, len_limit);
         if (paired_end == false && adp_1_len > 0)
             res = filter_adapter_seq(href, adapter_1, adp_1_len, adp_1_kmers,
-                                     kmer_size, distance, len_limit);
+                                     kmer_size, ::distance, len_limit);
         if (res <= 0) {
             // cerr << "Sequence " << href->seq << " contains adapter.\n";
             counter["adapter"]++;
@@ -754,6 +760,16 @@ print_results(int argc, char **argv,
     map<string, map<string, long> >::iterator it;
 
     string log_path = out_path + "process_radtags.log";
+    if (!in_path_1.empty()) {
+        // In directory mode, use `$out_path/process_radtags.$(basename $in_path).log`.
+        // For consistency we always use realpath().
+        char abspath [PATH_MAX];
+        realpath(in_path_1.c_str(), abspath);
+        string abspath_s (abspath);
+        size_t p = abspath_s.find_last_of('/');
+        string in_dir_name = abspath_s.substr(p+1);
+        log_path = out_path + "process_radtags." + in_dir_name + ".log";
+    }
     ofstream log(log_path.c_str());
 
     if (log.fail()) {
@@ -810,17 +826,13 @@ print_results(int argc, char **argv,
         c["retained"]     += it->second["retained"];
     }
 
-    std::ostream cerr_bis (cerr.rdbuf());
-    cerr_bis << std::fixed << std::setprecision(1);
-
-    auto print_nreads = [&cerr_bis,&c] (long n, const string& legend) {
-        size_t nspaces = std::to_string(c["total"]).length() - std::to_string(n).length();
-        cerr_bis << string(nspaces, ' ')
-             << n << " " << legend
-             << " (" << (double) n / c["total"] * 100 << "%)\n";
+    auto print_nreads = [&c] (long n, const string& legend) {
+        size_t nspaces = to_string(c["total"]).length() - to_string(n).length();
+        cerr << string(nspaces, ' ') << n << " " << legend
+             << " (" << as_percentage((double) n / c["total"]) << ")\n";
     };
 
-    cerr_bis << c["total"] << " total sequences\n";
+    cerr << c["total"] << " total sequences\n";
     if (filter_illumina)
         print_nreads(c["ill_filtered"], "failed Illumina filtered reads");
     if (filter_adapter)
@@ -861,8 +873,8 @@ print_results(int argc, char **argv,
     if (bc_names)
         log << "Filename\t";
     log << "Total\t"
-        << "No RadTag\t"
-        << "Low Quality\t"
+        << "NoRadTag\t"
+        << "LowQuality\t"
         << "Retained\n";
 
     set<BarcodePair> barcode_list;
@@ -915,7 +927,6 @@ int  compare_barcodes(pair<BarcodePair, int> a, pair<BarcodePair, int> b) {
 }
 
 int parse_command_line(int argc, char* argv[]) {
-    FileT ftype;
     int c;
 
     while (1) {
@@ -1007,22 +1018,18 @@ int parse_command_line(int argc, char* argv[]) {
             break;
         case 'f':
             in_file = optarg;
-            ftype   = FileT::fastq;
             break;
         case 'p':
             in_path_1 = optarg;
             in_path_2 = in_path_1;
-            ftype     = FileT::fastq;
             break;
         case '1':
             paired     = true;
             in_file_p1 = optarg;
-            ftype      = FileT::fastq;
             break;
         case '2':
             paired     = true;
             in_file_p2 = optarg;
-            ftype      = FileT::fastq;
             break;
         case 'P':
             paired = true;
@@ -1053,9 +1060,11 @@ int parse_command_line(int argc, char* argv[]) {
             break;
         case 'e':
             renz_1 = optarg;
+            renz_1.at(0) = tolower(renz_1.at(0));
             break;
         case 'z':
             renz_2 = optarg;
+            renz_2.at(0) = tolower(renz_2.at(0));
             break;
         case 'b':
             barcode_file = optarg;
@@ -1063,7 +1072,7 @@ int parse_command_line(int argc, char* argv[]) {
                 barcode_type = inline_null;
             break;
         case 'm':
-            merge = true;
+            ::merge = true;
             break;
         case 'D':
             discards = true;
@@ -1106,7 +1115,7 @@ int parse_command_line(int argc, char* argv[]) {
             filter_adapter = true;
             break;
         case 'T':
-            distance = is_integer(optarg);
+            ::distance = is_integer(optarg);
             break;
         case 'H':
             retain_header = true;
@@ -1135,6 +1144,11 @@ int parse_command_line(int argc, char* argv[]) {
         }
     }
 
+    if (optind < argc) {
+        cerr << "Error: Failed to parse command line: '" << argv[optind] << "' is seen as a positional argument. Expected no positional arguments.\n";
+        help();
+    }
+
     if (in_file.length() == 0 && in_path_1.length() == 0 && in_file_p1.length() == 0) {
         cerr << "You must specify an input file of a directory path to a set of input files.\n";
         help();
@@ -1167,8 +1181,16 @@ int parse_command_line(int argc, char* argv[]) {
     if (out_path.at(out_path.length() - 1) != '/')
         out_path += "/";
 
-    if (in_file_type == FileT::unknown)
-        in_file_type = ftype;
+    if (in_file_type == FileT::unknown) {
+        if (!in_file.empty())
+            in_file_type = guess_file_type(in_file);
+        else if (!in_file_p1.empty())
+            in_file_type = guess_file_type(in_file_p1);
+        // In directory mode, just use the default.
+
+        if (in_file_type == FileT::unknown)
+            in_file_type = FileT::gzfastq;
+    }
 
     if (in_file_type == FileT::bam && paired == true && interleaved == false) {
         cerr << "You may only specify a BAM input file for paired-end data if the read pairs are interleaved.\n";
@@ -1188,7 +1210,7 @@ int parse_command_line(int argc, char* argv[]) {
     if (barcode_file.length() == 0)
         cerr << "No barcodes specified, files will not be demultiplexed.\n";
 
-    if (barcode_file.length() > 0 && merge) {
+    if (barcode_file.length() > 0 && ::merge) {
         cerr << "You may specify a set of barcodes, or that all files should be merged, not both.\n";
         help();
     }
@@ -1234,26 +1256,29 @@ void version() {
 
 void help() {
     std::cerr << "process_radtags " << VERSION << "\n"
-              << "process_radtags [-f in_file | -p in_dir [-P] [-I] | -1 pair_1 -2 pair_2] -b barcode_file -o out_dir -e enz [-c] [-q] [-r] [-t len] [-D] [-w size] [-s lim] [-h]\n"
-              << "  f: path to the input file if processing single-end sequences.\n"
-              << "  i: input file type, either 'bustard' for the Illumina BUSTARD format, 'bam', 'fastq' (default), or 'gzfastq' for gzipped FASTQ.\n"
-              << "  y: output type, either 'fastq', 'gzfastq', 'fasta', or 'gzfasta' (default is to match the input file type).\n"
+              << "process_radtags -p in_dir [--paired [--interleaved]] [-i format] -b barcode_file -o out_dir -e enz [-c] [-q] [-r] [-t len] [-D] [-w size] [-s lim]\n"
+              << "process_radtags -f in_file [-i format] -b barcode_file -o out_dir -e enz [-c] [-q] [-r] [-t len] [-D] [-w size] [-s lim]\n"
+              << "process_radtags -1 pair_1 -2 pair_2 [-i format] -b barcode_file -o out_dir -e enz [-c] [-q] [-r] [-t len] [-D] [-w size] [-s lim]\n"
+              << "\n"
               << "  p: path to a directory of files.\n"
-              << "  P: files contained within directory specified by '-p' are paired.\n"
-              << "  I: specify that the paired-end reads are interleaved in single files.\n"
+              << "  P,--paired: files contained within the directory are paired.\n"
+              << "  I,--interleaved: specify that the paired-end reads are interleaved in single files.\n"
+              << "  i: input file type, either 'fastq', 'gzfastq' (gzipped fastq), 'bam', or 'bustard' (default: guess, or gzfastq if unable to).\n"
+              << "  b: path to a file containing barcodes for this run.\n"
+              << "  o: path to output the processed files.\n"
+              << "  f: path to the input file if processing single-end sequences.\n"
               << "  1: first input file in a set of paired-end sequences.\n"
               << "  2: second input file in a set of paired-end sequences.\n"
-              << "  o: path to output the processed files.\n"
-              << "  b: path to a file containing barcodes for this run.\n"
               << "  c: clean data, remove any read with an uncalled base.\n"
               << "  q: discard reads with low quality scores.\n"
               << "  r: rescue barcodes and RAD-Tags.\n"
               << "  t: truncate final read length to this value.\n"
-              << "  E: specify how quality scores are encoded, 'phred33' (Illumina 1.8+, Sanger, default) or 'phred64' (Illumina 1.3 - 1.5).\n"
               << "  D: capture discarded reads to a file.\n"
+              << "  E: specify how quality scores are encoded, 'phred33' (Illumina 1.8+/Sanger, default) or 'phred64' (Illumina 1.3-1.5).\n"
               << "  w: set the size of the sliding window as a fraction of the read length, between 0 and 1 (default 0.15).\n"
               << "  s: set the score limit. If the average score within the sliding window drops below this value, the read is discarded (default 10).\n"
-              << "  h: display this help messsage." << "\n\n"
+              << "  y: output type, either 'fastq', 'gzfastq', 'fasta', or 'gzfasta' (default: match input type).\n"
+              << "\n"
               << "  Barcode options:\n"
               << "    --inline_null:   barcode is inline with sequence, occurs only on single-end read (default).\n"
               << "    --index_null:    barcode is provded in FASTQ header (Illumina i5 or i7 read).\n"
@@ -1261,7 +1286,8 @@ void help() {
               << "    --inline_inline: barcode is inline with sequence, occurs on single and paired-end read.\n"
               << "    --index_index:   barcode is provded in FASTQ header (Illumina i5 and i7 reads).\n"
               << "    --inline_index:  barcode is inline with sequence on single-end read and occurs in FASTQ header (from either i5 or i7 read).\n"
-              << "    --index_inline:  barcode occurs in FASTQ header (Illumina i5 or i7 read) and is inline with single-end sequence (for single-end data) on paired-end read (for paired-end data).\n\n"
+              << "    --index_inline:  barcode occurs in FASTQ header (Illumina i5 or i7 read) and is inline with single-end sequence (for single-end data) on paired-end read (for paired-end data).\n"
+              << "\n"
               << "  Restriction enzyme options:\n"
               << "    -e <enz>, --renz_1 <enz>: provide the restriction enzyme used (cut site occurs on single-end read)\n"
               << "    --renz_2 <enz>: if a double digest was used, provide the second restriction enzyme used (cut site occurs on the paired-end read).\n"
@@ -1283,6 +1309,7 @@ void help() {
 
         it++;
     }
+    cerr << "\n";
 
     std::cerr << "\n"
               << "  Adapter options:\n"
