@@ -24,6 +24,7 @@
 //
 
 #include "MetaPopInfo.h"
+#include "catalog_utils.h"
 
 #include "rxstacks.h"
 
@@ -31,7 +32,7 @@ typedef MetaPopInfo::Sample Sample;
 
 // Global variables to hold command-line options.
 int    num_threads = 1;
-int    batch_id    = 0;
+int    batch_id    = -1;
 string in_path;
 string out_path;
 FileT  in_file_type      = FileT::sql;
@@ -58,6 +59,7 @@ double homozygote_limit   =  2.71;
 const int barcode_size    = 5;
 
 int main (int argc, char* argv[]) {
+    IF_NDEBUG_TRY
 
     parse_command_line(argc, argv);
 
@@ -95,10 +97,6 @@ int main (int argc, char* argv[]) {
 
     MetaPopInfo mpopi;
     mpopi.init_directory(in_path);
-    if (mpopi.samples().empty()) {
-        cerr << "Error: Failed to find sample files in directory '" << in_path << "'.\n";
-        return -1;
-    }
 
     //
     // Open and initialize the log files.
@@ -397,6 +395,7 @@ int main (int argc, char* argv[]) {
         map<int, Locus *>::iterator stack_it;
         for (stack_it = stacks.begin(); stack_it != stacks.end(); stack_it++)
             delete stack_it->second;
+        stacks.clear();
         cerr << "done.\n";
     }
 
@@ -407,7 +406,22 @@ int main (int argc, char* argv[]) {
         log_hap_fh.close();
     }
 
+    //
+    // Free memory associated with the catalog and matches to the catalog.
+    //
+    for (map<int, CSLocus *>::iterator cat_it = catalog.begin(); cat_it != catalog.end(); cat_it++)
+        delete cat_it->second;
+    catalog.clear();
+
+    for (uint i = 0; i < catalog_matches.size(); i++) {
+        for (uint j = 0; j < catalog_matches[i].size(); j++)
+            delete catalog_matches[i][j];
+        catalog_matches[i].clear();
+    }
+    
+    cerr << "rxstacks is done.\n";
     return 0;
+    IF_NDEBUG_CATCH_ALL_EXCEPTIONS
 }
 
 int
@@ -472,12 +486,12 @@ calc_lnl_means(map<int, CSLocus *> &catalog, PopMap<CSLocus> *pmap)
             mean += d[i]->lnl;
         }
 
-        if (lnls.size() == 0) continue;
+        if (lnls.size() == 0)
+            continue;
 
         sort(lnls.begin(), lnls.end());
-
         mid    = lnls.size() / 2;
-        median = lnls.size() % 2 == 0 ? lnls[mid] + lnls[mid+1] / 2.0 : lnls[mid+1];
+        median = lnls.size() % 2 == 0 ? lnls[mid-1] + lnls[mid] / 2.0 : lnls[mid];
         mean   = mean / (double) lnls.size();
 
         cloc->lnl = mean;
@@ -551,14 +565,12 @@ prune_mst_haplotypes(CSLocus *cloc, Datum *d, Locus *loc, unsigned long &pruned_
     MinSpanTree *mst = new MinSpanTree;
 
     map<string, int>::iterator it;
-    vector<uint>   keys;
     vector<string> haps;
     Node *n;
 
     for (it = cloc->hap_cnts.begin(); it != cloc->hap_cnts.end(); it++) {
-        n = mst->add_node(it->first);
+        mst->add_node(it->first);
         haps.push_back(it->first);
-        keys.push_back(n->id);
     }
 
     //
@@ -609,8 +621,10 @@ prune_mst_haplotypes(CSLocus *cloc, Datum *d, Locus *loc, unsigned long &pruned_
     //
     sort(haplotypes.begin(), haplotypes.end(), compare_pair_haplotype_rev);
 
-    if (size <= 2)
+    if (size <= 2) {
+        delete mst;
         return 0;
+    }
 
     //
     // Pull out the two most frequently occuring haplotypes.
@@ -720,6 +734,8 @@ prune_mst_haplotypes(CSLocus *cloc, Datum *d, Locus *loc, unsigned long &pruned_
     //
     generate_matched_haplotypes(cloc, loc, d);
 
+    delete mst;
+
     return 0;
 }
 
@@ -1662,10 +1678,6 @@ parse_command_line(int argc, char* argv[])
             break;
         case 'b':
             batch_id = is_integer(optarg);
-            if (batch_id < 0) {
-                cerr << "Batch ID (-b) must be an integer, e.g. 1, 2, 3\n";
-                help();
-            }
             break;
         case 'o':
             out_path = optarg;
@@ -1674,7 +1686,7 @@ parse_command_line(int argc, char* argv[])
             if (strcmp(optarg, "snp") == 0) {
                 model_type = snp;
             } else if (strcmp(optarg, "fixed") == 0) {
-                model_type = fixed;
+                model_type = ::fixed;
             } else if (strcmp(optarg, "bounded") == 0) {
                 model_type = bounded;
             } else {
@@ -1729,14 +1741,19 @@ parse_command_line(int argc, char* argv[])
         }
     }
 
+    if (optind < argc) {
+        cerr << "Error: Failed to parse command line: '" << argv[optind] << "' is seen as a positional argument. Expected no positional arguments.\n";
+        help();
+    }
+
     if (in_path.length() == 0) {
         cerr << "You must specify a path to the directory containing Stacks output files.\n";
         help();
     }
 
     if (out_path.length() == 0) {
-        cerr << "No output path specified, files in '" << in_path << "' will be overwritten.\n";
-        out_path = in_path;
+        cerr << "You must specify a path to a directory where to write output files.\n";
+        help();
     }
 
     if (in_path.at(in_path.length() - 1) != '/')
@@ -1745,9 +1762,17 @@ parse_command_line(int argc, char* argv[])
     if (out_path.at(out_path.length() - 1) != '/')
         out_path += "/";
 
-    if (batch_id == 0) {
-        cerr << "You must specify a batch ID.\n";
-        help();
+    if (batch_id < 0) {
+        vector<int> cat_ids = find_catalogs(in_path);
+        if (cat_ids.size() == 1) {
+            batch_id = cat_ids[0];
+        } else if (cat_ids.empty()) {
+            cerr << "Error: Unable to find a catalog in '" << in_path << "'.\n";
+            help();
+        } else {
+            cerr << "Error: Input directory contains several catalogs, please specify -b.\n";
+            help();
+        }
     }
 
     if (alpha != 0.1 && alpha != 0.05 && alpha != 0.01 && alpha != 0.001) {
@@ -1785,13 +1810,11 @@ void version() {
 
 void help() {
     std::cerr << "rxstacks " << VERSION << "\n"
-              << "rxstacks -b batch_id -P path [-o path] [-t threads] [-v] [-h]" << "\n"
-              << "  b: Batch ID to examine when exporting from the catalog.\n"
+              << "rxstacks -P path -o path [-t threads] [-b batch_id]" << "\n"
               << "  P: path to the Stacks output files.\n"
-              << "  o: output path to write results.\n"
+              << "  o: output path to write results ('.' to override the current files).\n"
               << "  t: number of threads to run in parallel sections of code.\n"
-              << "  v: print program version." << "\n"
-              << "  h: display this help messsage." << "\n\n"
+              << "  b: database/batch ID of the input catalog to consider (default: guess).\n"
               << "  Filtering options:\n"
               << "    --lnl_filter: filter catalog loci based on the mean log likelihood of the catalog locus in the population.\n"
               << "      --lnl_lim <limit>: minimum log likelihood required to keep a catalog locus.\n"
