@@ -29,13 +29,13 @@
 //
 #include "stacks.h"
 
-Rem::Rem() { 
+Rem::Rem() {
     this->id         = 0;
-    this->seq        = NULL; 
+    this->seq        = NULL;
     this->utilized   = false;
 }
 
-Rem::Rem(int id, uint seq_id, DNANSeq *seq) { 
+Rem::Rem(int id, uint seq_id, DNANSeq *seq) {
     this->id       = id;
     this->utilized = false;
 
@@ -86,7 +86,7 @@ int PStack::add_seq(const char *seq) {
     return 0;
 }
 
-int PStack::add_seq(DNANSeq *seq) {
+int PStack::add_seq(const DNANSeq *seq) {
     if (this->seq != NULL)
         delete this->seq;
 
